/****************************************************************************/
/*! @file
@brief DirectShow

obt@w_O
-----------------------------------------------------------------------------
	Copyright (C) 2004 T.Imoto
-----------------------------------------------------------------------------
@author		T.Imoto
@date		2004/08/05
@note
			2004/08/05	T.Imoto		
*****************************************************************************/

#ifndef __VIDEO_BUFFER_RENDER_H__
#define __VIDEO_BUFFER_RENDER_H__

#include "IBufferRenderer.h"
#include "IRendererBufferAccess.h"
#include "IRendererBufferVideo.h"

#define EC_UPDATE		(EC_USER+1)

class TBufferRendererAllocator;
class TBufferRendererInputPin;
class TBufferRenderer;

//----------------------------------------------------------------------------
//! @brief Buffer Renderer̃AP[^[
//----------------------------------------------------------------------------
class TBufferRendererAllocator : public CBaseAllocator
{
private:
	CMediaSample	*m_pMediaSample;	//!< ݂̃fBATvւ̃|C^
	TBufferRenderer	*m_pRenderer;		//!< ̃AP[^[_[ւ̃|C^

protected:
	void Free(void);
	HRESULT Alloc(void);

public:
	TBufferRendererAllocator( TBufferRenderer *, TCHAR *, LPUNKNOWN, HRESULT * );
	TBufferRendererAllocator( TCHAR *, LPUNKNOWN, HRESULT * );
	virtual ~TBufferRendererAllocator();

	// I[o[Ch
	STDMETHODIMP SetProperties( ALLOCATOR_PROPERTIES* pRequest, ALLOCATOR_PROPERTIES* pActual );

	// _uobt@Op|C^ւ\bh
	void SetPointer( IMediaSample *media, BYTE *ptr );
	void SetPointer( BYTE *ptr ) { SetPointer(m_pMediaSample,ptr); };
};

//----------------------------------------------------------------------------
//! @brief Buffer Renderer̃Cvbgs
//----------------------------------------------------------------------------
class TBufferRendererInputPin : public CRendererInputPin
{
private:
	TBufferRenderer	*m_pRenderer;		//!< ̃s_[ւ̃|C^
	CCritSec		*m_pInterfaceLock;	//!< bNp\[X
	bool			m_ActiveAllocator;	//!< ̃s_[̃AP[^[gĂ邩ǂ

public:
	TBufferRendererInputPin( TBufferRenderer *pRenderer, CCritSec *pInterfaceLock, HRESULT *phr, LPCWSTR name );
	virtual ~TBufferRendererInputPin();
	bool ActiveAllocator( void ) const;

	//I[o[Ch
	STDMETHODIMP GetAllocator( IMemAllocator **ppAllocator );
	STDMETHODIMP NotifyAllocator( IMemAllocator * pAllocator, BOOL bReadOnly );

	// _uobt@Op|C^ւ\bh
	void SetPointer( IMediaSample *media, BYTE *ptr );
	void SetPointer( BYTE *ptr );
};

//----------------------------------------------------------------------------
//! @brief obt@w_O
//----------------------------------------------------------------------------
class TBufferRenderer : public CBaseVideoRenderer, public IRendererBufferAccess, public IRendererBufferVideo
{
	friend class TBufferRendererInputPin;
	friend class TBufferRendererAllocator;

private:
	long	m_VideoWidth;		//!< rfI̕
	long	m_VideoHeight;		//!< rfI̍
	long	m_VideoPitch;		//!< rfĨsb`(1s̃oCg)
	BYTE	*m_Buffer[2];		//!< _Oobt@ւ̃|C^
	bool	m_IsBufferOwner[2];	//!< obt@̃NXɊ蓖Ăꂽ̂ǂێ
	int		m_FrontBuffer;		//!< ݂̃tgobt@ǂ炩ێ
	LONG	m_StartFrame;
	LONG	m_StopFrame;

	TBufferRendererInputPin		m_InputPin;		//!< ̓s
	TBufferRendererAllocator	m_Allocator;	//!< ̊蓖

	CCritSec		m_BufferLock;	//!< obt@փANZX鎞ɃbN
	CMediaType		m_MtIn;			//!< Source connection media type

public:
	static CUnknown * WINAPI CreateInstance( LPUNKNOWN, HRESULT * );
	TBufferRenderer( TCHAR *pName, LPUNKNOWN pUnk, HRESULT *phr );
	~TBufferRenderer();

	DECLARE_IUNKNOWN
	STDMETHODIMP NonDelegatingQueryInterface( REFIID riid, void **ppv );

	HRESULT OnStartStreaming(void);
	void OnRenderStart( IMediaSample *pMediaSample );

// IRendererBufferAccess
	STDMETHOD( SetFrontBuffer )( BYTE *buff, long *size);
	STDMETHOD( SetBackBuffer ) ( BYTE *buff, long *size);
	STDMETHOD( GetFrontBuffer )( BYTE **buff, long *size);
	STDMETHOD( GetBackBuffer ) ( BYTE **buff, long *size);

// IRendererBufferVideo
	virtual /* [propget] */ HRESULT STDMETHODCALLTYPE get_AvgTimePerFrame( 
			/* [retval][out] */ REFTIME *pAvgTimePerFrame);
	virtual /* [propget] */ HRESULT STDMETHODCALLTYPE get_VideoWidth( 
			/* [retval][out] */ long *pVideoWidth);
	virtual /* [propget] */ HRESULT STDMETHODCALLTYPE get_VideoHeight( 
			/* [retval][out] */ long *pVideoHeight);

protected:
	// I[o[Ch
	// DShowɃR[
	HRESULT CheckMediaType( const CMediaType *pmt );		// Format acceptable?
	HRESULT DoRenderSample( IMediaSample *pMediaSample );	// New video sample
	HRESULT SetMediaType( const CMediaType *pmt );			// Video format notification
	
	void SwapBuffer( IMediaSample *pSample );

	void AllocFrontBuffer( size_t size );
	void AllocBackBuffer( size_t size );

	void FreeFrontBuffer();
	void FreeBackBuffer();

	bool IsAllocatedFrontBuffer() { return ((m_FrontBuffer==1) ? (m_Buffer[1] != NULL) : (m_Buffer[0] != NULL)); }
	bool IsAllocatedBackBuffer() { return ((m_FrontBuffer==1) ? (m_Buffer[0] != NULL) : (m_Buffer[1] != NULL)); }
	
	long GetBufferSize() { return (m_VideoHeight * m_VideoPitch); }

	void SetFrontBuffer( BYTE *buff );
	void SetBackBuffer( BYTE *buff );
	BYTE *GetFrontBuffer();
	BYTE *GetBackBuffer();

	//! _uobt@Op|C^ւ\bh
	void SetPointer( IMediaSample *media, BYTE *ptr )
	{
		m_InputPin.SetPointer( media, ptr );
	}
	void SetPointer( BYTE *ptr )
	{
		m_InputPin.SetPointer( ptr );
	}
};


#endif	// __VIDEO_BUFFER_RENDER_H__
